=== modified file '.travis.yml'
--- .travis.yml	2020-02-07 02:14:30 +0000
+++ .travis.yml	2020-02-13 23:57:28 +0000
@@ -13,9 +13,11 @@
   include:
     - python: 3.7
       dist: xenial
+    - python: 3.8
+      dist: xenial
 
 script:
-  - make docs extensions
+  - make docs extensions PYTHON=python
   - BRZ_PLUGIN_PATH=-site:-user python -Werror -Wignore::ImportWarning -Wignore::PendingDeprecationWarning -Wignore::DeprecationWarning -Wignore::ResourceWarning -Wignore::UserWarning ./brz selftest --parallel=fork $SELFTEST_OPTIONS
 
 install:

=== modified file 'breezy/bzr/bundle/bundle_data.py'
--- breezy/bzr/bundle/bundle_data.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/bundle/bundle_data.py	2020-02-13 23:57:28 +0000
@@ -45,8 +45,8 @@
 from ..testament import StrictTestament
 from ...trace import mutter, warning
 from ...tree import (
+    InterTree,
     Tree,
-    find_previous_path,
     )
 from ..xml5 import serializer_v5
 
@@ -489,6 +489,7 @@
         self.deleted = []
         self.revision_id = revision_id
         self._inventory = None
+        self._base_inter = InterTree.get(self.base_tree, self)
 
     def __str__(self):
         return pprint.pformat(self.__dict__)
@@ -618,7 +619,7 @@
                 in the text-store, so that the file contents would
                 then be cached.
         """
-        old_path = find_previous_path(self, self.base_tree, path)
+        old_path = self._base_inter.find_source_path(path)
         if old_path is None:
             patch_original = None
         else:

=== modified file 'breezy/bzr/bundle/serializer/v4.py'
--- breezy/bzr/bundle/serializer/v4.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/bundle/serializer/v4.py	2020-02-13 23:57:28 +0000
@@ -180,9 +180,9 @@
         """
         name = self.encode_name(repo_kind, revision_id, file_id)
         encoded_metadata = bencode.bencode(metadata)
-        self._container.add_bytes_record(encoded_metadata, [(name, )])
+        self._container.add_bytes_record([encoded_metadata], len(encoded_metadata), [(name, )])
         if metadata[b'storage_kind'] != b'header':
-            self._container.add_bytes_record(bytes, [])
+            self._container.add_bytes_record([bytes], len(bytes), [])
 
 
 class BundleReader(object):

=== modified file 'breezy/bzr/groupcompress.py'
--- breezy/bzr/groupcompress.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/groupcompress.py	2020-02-13 23:57:28 +0000
@@ -270,7 +270,7 @@
         :return: The bytes for the content
         """
         if start == end == 0:
-            return b''
+            return []
         self._ensure_content(end)
         # The bytes are 'f' or 'd' for the type, then a variable-length
         # base128 integer for the content size, then the actual content
@@ -291,9 +291,9 @@
             raise ValueError('end != len according to field header'
                              ' %s != %s' % (end, content_start + content_len))
         if c == b'f':
-            return self._content[content_start:end]
+            return [self._content[content_start:end]]
         # Must be type delta as checked above
-        return apply_delta_to_source(self._content, content_start, end)
+        return [apply_delta_to_source(self._content, content_start, end)]
 
     def set_chunked_content(self, content_chunks, length):
         """Set the content of this block to the given chunks."""
@@ -442,7 +442,7 @@
         #       get_bytes_as call? After Manager.get_record_stream() returns
         #       the object?
         self._manager = manager
-        self._bytes = None
+        self._chunks = None
         self.storage_kind = 'groupcompress-block'
         if not first:
             self.storage_kind = 'groupcompress-block-ref'
@@ -463,7 +463,7 @@
         except zlib.error as value:
             raise DecompressCorruption("zlib: " + str(value))
         block = self._manager._block
-        self._bytes = block.extract(self.key, self._start, self._end)
+        self._chunks = block.extract(self.key, self._start, self._end)
         # There are code paths that first extract as fulltext, and then
         # extract as storage_kind (smart fetch). So we don't break the
         # refcycle here, but instead in manager.get_record_stream()
@@ -476,24 +476,24 @@
             else:
                 return b''
         if storage_kind in ('fulltext', 'chunked', 'lines'):
-            if self._bytes is None:
+            if self._chunks is None:
                 self._extract_bytes()
             if storage_kind == 'fulltext':
-                return self._bytes
+                return b''.join(self._chunks)
             elif storage_kind == 'chunked':
-                return [self._bytes]
+                return self._chunks
             else:
-                return osutils.split_lines(self._bytes)
+                return osutils.chunks_to_lines(self._chunks)
         raise errors.UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
     def iter_bytes_as(self, storage_kind):
-        if self._bytes is None:
+        if self._chunks is None:
             self._extract_bytes()
         if storage_kind == 'chunked':
-            return iter([self._bytes])
+            return iter(self._chunks)
         elif storage_kind == 'lines':
-            return osutils.split_lines(self._bytes)
+            return iter(osutils.chunks_to_lines(self._chunks))
         raise errors.UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
@@ -906,7 +906,7 @@
         """Extract a key previously added to the compressor.
 
         :param key: The key to extract.
-        :return: An iterable over bytes and the sha1.
+        :return: An iterable over chunks and the sha1.
         """
         (start_byte, start_chunk, end_byte,
          end_chunk) = self.labels_deltas[key]
@@ -920,7 +920,7 @@
                 raise ValueError('Index claimed fulltext len, but stored bytes'
                                  ' claim %s != %s'
                                  % (len(stored_bytes), data_len))
-            data = stored_bytes[offset + 1:]
+            data = [stored_bytes[offset + 1:]]
         else:
             if kind != b'd':
                 raise ValueError('Unknown content kind, bytes claim %s' % kind)
@@ -932,8 +932,8 @@
                 raise ValueError('Index claimed delta len, but stored bytes'
                                  ' claim %s != %s'
                                  % (len(stored_bytes), data_len))
-            data = apply_delta(source, stored_bytes[offset + 1:])
-        data_sha1 = osutils.sha_string(data)
+            data = [apply_delta(source, stored_bytes[offset + 1:])]
+        data_sha1 = osutils.sha_strings(data)
         return data, data_sha1
 
     def flush(self):
@@ -1656,9 +1656,9 @@
                         # self._compressor.
                         for factory in batcher.yield_factories(full_flush=True):
                             yield factory
-                        bytes, sha1 = self._compressor.extract(key)
+                        chunks, sha1 = self._compressor.extract(key)
                         parents = self._unadded_refs[key]
-                        yield FulltextContentFactory(key, parents, sha1, bytes)
+                        yield ChunkedContentFactory(key, parents, sha1, chunks)
                         continue
                     if batcher.add_key(key) > BATCH_SIZE:
                         # Ok, this batch is big enough.  Yield some results.

=== modified file 'breezy/bzr/inventory_delta.py'
--- breezy/bzr/inventory_delta.py	2019-02-02 15:13:30 +0000
+++ breezy/bzr/inventory_delta.py	2020-02-09 01:16:06 +0000
@@ -277,23 +277,25 @@
         else:
             raise InventoryDeltaError("value %(val)r is not a bool", val=value)
 
-    def parse_text_bytes(self, bytes):
+    def parse_text_bytes(self, lines):
         """Parse the text bytes of a serialized inventory delta.
 
         If versioned_root and/or tree_references flags were set via
         require_flags, then the parsed flags must match or a BzrError will be
         raised.
 
-        :param bytes: The bytes to parse. This can be obtained by calling
-            delta_to_lines and then doing ''.join(delta_lines).
+        :param lines: The lines to parse. This can be obtained by calling
+            delta_to_lines.
         :return: (parent_id, new_id, versioned_root, tree_references,
             inventory_delta)
         """
-        if not bytes.endswith(b'\n'):
-            last_line = bytes.rsplit(b'\n', 1)[-1]
-            raise InventoryDeltaError(
-                'last line not empty: %(line)r', line=last_line)
-        lines = bytes.split(b'\n')[:-1]  # discard the last empty line
+        if not lines:
+            raise InventoryDeltaError(
+                'inventory delta is empty')
+        if not lines[-1].endswith(b'\n'):
+            raise InventoryDeltaError(
+                'last line not empty: %(line)r', line=lines[-1])
+        lines = [line.rstrip(b'\n') for line in lines]  # discard the last empty line
         if not lines or lines[0] != b'format: %s' % FORMAT_1:
             raise InventoryDeltaError(
                 'unknown format %(line)r', line=lines[0:1])

=== modified file 'breezy/bzr/knitpack_repo.py'
--- breezy/bzr/knitpack_repo.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/knitpack_repo.py	2020-02-13 23:57:28 +0000
@@ -683,7 +683,7 @@
                 else:
                     df, _ = knit._parse_record_header(key, raw_data)
                     df.close()
-                pos, size = writer.add_bytes_record(raw_data, names)
+                pos, size = writer.add_bytes_record([raw_data], len(raw_data), names)
                 write_index.add_node(key, eol_flag + b"%d %d" % (pos, size))
                 pb.update("Copied record", record_index)
                 record_index += 1
@@ -735,7 +735,7 @@
                     # check the header only
                     df, _ = knit._parse_record_header(key, raw_data)
                     df.close()
-                pos, size = writer.add_bytes_record(raw_data, names)
+                pos, size = writer.add_bytes_record([raw_data], len(raw_data), names)
                 write_index.add_node(key, eol_flag + b"%d %d" %
                                      (pos, size), references)
                 pb.update("Copied record", record_index)

=== modified file 'breezy/bzr/pack.py'
--- breezy/bzr/pack.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/pack.py	2020-02-13 23:57:28 +0000
@@ -139,10 +139,11 @@
         """Finish writing a container."""
         self.write_func(self._serialiser.end())
 
-    def add_bytes_record(self, bytes, names):
+    def add_bytes_record(self, chunks, length, names):
         """Add a Bytes record with the given names.
 
-        :param bytes: The bytes to insert.
+        :param bytes: The chunks to insert.
+        :param length: Total length of bytes in chunks
         :param names: The names to give the inserted bytes. Each name is
             a tuple of bytestrings. The bytestrings may not contain
             whitespace.
@@ -154,13 +155,13 @@
             and thus are only suitable for use by a ContainerReader.
         """
         current_offset = self.current_offset
-        length = len(bytes)
         if length < self._JOIN_WRITES_THRESHOLD:
             self.write_func(self._serialiser.bytes_header(length, names)
-                            + bytes)
+                            + b''.join(chunks))
         else:
             self.write_func(self._serialiser.bytes_header(length, names))
-            self.write_func(bytes)
+            for chunk in chunks:
+                self.write_func(chunk)
         self.records_written += 1
         # return a memo of where we wrote data to allow random access.
         return current_offset, self.current_offset - current_offset

=== modified file 'breezy/bzr/pack_repo.py'
--- breezy/bzr/pack_repo.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/pack_repo.py	2020-02-13 23:57:28 +0000
@@ -1974,7 +1974,7 @@
             object supplied to the PackAccess object.
         """
         p_offset, p_length = self._container_writer.add_bytes_record(
-            b''.join(raw_data), [])
+            raw_data, size, [])
         return (self._write_index, p_offset, p_length)
 
     def add_raw_records(self, key_sizes, raw_data):

=== modified file 'breezy/bzr/remote.py'
--- breezy/bzr/remote.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/remote.py	2020-02-13 23:57:28 +0000
@@ -2039,7 +2039,7 @@
                 "Unexpected stream %r received" % substream_kind)
         for record in substream:
             (parent_id, new_id, versioned_root, tree_references, invdelta) = (
-                deserializer.parse_text_bytes(record.get_bytes_as("fulltext")))
+                deserializer.parse_text_bytes(record.get_bytes_as("lines")))
             if parent_id != prev_inv.revision_id:
                 raise AssertionError("invalid base %r != %r" % (parent_id,
                                                                 prev_inv.revision_id))

=== modified file 'breezy/bzr/versionedfile.py'
--- breezy/bzr/versionedfile.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/versionedfile.py	2020-02-13 23:57:28 +0000
@@ -138,7 +138,7 @@
         elif storage_kind == 'lines':
             if self._chunks_are_lines:
                 return iter(self._chunks)
-            return osutils.chunks_to_lines(self._chunks)
+            return iter(osutils.chunks_to_lines(self._chunks))
         raise errors.UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 

=== modified file 'breezy/bzr/vf_repository.py'
--- breezy/bzr/vf_repository.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/vf_repository.py	2020-02-13 23:57:28 +0000
@@ -1932,7 +1932,7 @@
         target_tree_refs = self.target_repo._format.supports_tree_reference
         for record in substream:
             # Insert the delta directly
-            inventory_delta_bytes = record.get_bytes_as('fulltext')
+            inventory_delta_bytes = record.get_bytes_as('lines')
             deserialiser = inventory_delta.InventoryDeltaDeserializer()
             try:
                 parse_result = deserialiser.parse_text_bytes(

=== modified file 'breezy/bzr/workingtree_4.py'
--- breezy/bzr/workingtree_4.py	2020-02-07 02:14:30 +0000
+++ breezy/bzr/workingtree_4.py	2020-02-13 23:57:28 +0000
@@ -2044,6 +2044,8 @@
 
     def is_executable(self, path):
         inv, inv_file_id = self._path2inv_file_id(path)
+        if inv_file_id is None:
+            raise errors.NoSuchFile(path)
         ie = inv.get_entry(inv_file_id)
         if ie.kind != "file":
             return False

=== modified file 'breezy/commands.py'
--- breezy/commands.py	2020-02-07 02:14:30 +0000
+++ breezy/commands.py	2020-02-13 23:57:28 +0000
@@ -971,7 +971,11 @@
 def apply_coveraged(the_callable, *args, **kwargs):
     import coverage
     cov = coverage.Coverage()
-    os.environ['COVERAGE_PROCESS_START'] = cov.config_file
+    try:
+        config_file = cov.config.config_file
+    except AttributeError:  # older versions of coverage
+        config_file = cov.config_file
+    os.environ['COVERAGE_PROCESS_START'] = config_file
     cov.start()
     try:
         return exception_to_return_code(the_callable, *args, **kwargs)

=== modified file 'breezy/export_pot.py'
--- breezy/export_pot.py	2020-02-07 02:14:30 +0000
+++ breezy/export_pot.py	2020-02-13 23:57:28 +0000
@@ -30,6 +30,7 @@
 
 import inspect
 import os
+import sys
 
 import breezy
 from . import (
@@ -86,7 +87,7 @@
             # string terminates on. It's more useful to have the line the
             # string begins on. Unfortunately, counting back newlines is
             # only an approximation as the AST is ignorant of escaping.
-            str_to_lineno[node.s] = node.lineno - node.s.count('\n')
+            str_to_lineno[node.s] = node.lineno - (0 if sys.version_info >= (3, 8) else node.s.count('\n'))
     return cls_to_lineno, str_to_lineno
 
 

=== modified file 'breezy/git/annotate.py'
--- breezy/git/annotate.py	2020-01-30 18:07:32 +0000
+++ breezy/git/annotate.py	2020-02-09 01:27:02 +0000
@@ -70,7 +70,7 @@
 
     def iter_bytes_as(self, storage_kind):
         if storage_kind == 'lines':
-            return osutils.chunks_to_lines(self.store[self.blob_id].as_raw_chunks())
+            return iter(osutils.chunks_to_lines(self.store[self.blob_id].as_raw_chunks()))
         elif storage_kind == 'chunked':
             return iter(self.store[self.blob_id].as_raw_chunks())
         raise UnavailableRepresentation(self.key, storage_kind,

=== modified file 'breezy/git/fetch.py'
--- breezy/git/fetch.py	2020-02-07 02:14:30 +0000
+++ breezy/git/fetch.py	2020-02-13 23:57:28 +0000
@@ -54,7 +54,7 @@
 from ..bzr.testament import (
     StrictTestament3,
     )
-from ..tree import find_previous_path
+from ..tree import InterTree
 from ..tsort import (
     topo_sort,
     )
@@ -125,7 +125,11 @@
     # Check what revision we should store
     parent_keys = []
     for ptree in parent_bzr_trees:
-        ppath = find_previous_path(base_bzr_tree, ptree, decoded_path, file_id, recurse='none')
+        intertree = InterTree.get(ptree, base_bzr_tree)
+        try:
+            ppath = intertree.find_source_paths(decoded_path, recurse='none')
+        except errors.NoSuchFile:
+            continue
         if ppath is None:
             continue
         pkind = ptree.kind(ppath)

=== modified file 'breezy/git/object_store.py'
--- breezy/git/object_store.py	2020-02-07 02:14:30 +0000
+++ breezy/git/object_store.py	2020-02-13 23:57:28 +0000
@@ -42,11 +42,11 @@
     osutils,
     ui,
     )
-from ..tree import find_previous_path
 from ..lock import LogicalLockResult
 from ..revision import (
     NULL_REVISION,
     )
+from ..tree import InterTree
 from ..bzr.testament import (
     StrictTestament3,
     )
@@ -230,7 +230,8 @@
 
     def find_unchanged_parent_ie(path, kind, other, parent_trees):
         for ptree in parent_trees:
-            ppath = find_previous_path(tree, ptree, path)
+            intertree = InterTree.get(ptree, tree)
+            ppath = intertree.find_source_path(path)
             if ppath is not None:
                 pkind = ptree.kind(ppath)
                 if kind == "file":

=== modified file 'breezy/git/remote.py'
--- breezy/git/remote.py	2020-02-07 02:14:30 +0000
+++ breezy/git/remote.py	2020-02-13 23:57:28 +0000
@@ -48,6 +48,7 @@
     PermissionDenied,
     UninitializableFormat,
     )
+from ..revision import NULL_REVISION
 from ..revisiontree import RevisionTree
 from ..transport import (
     Transport,
@@ -444,10 +445,10 @@
         else:
             pb = None
 
-        def get_changed_refs_wrapper(refs):
-            # TODO(jelmer): This drops symref information
-            self._refs = remote_refs_dict_to_container(refs)
-            return get_changed_refs(refs)
+        def get_changed_refs_wrapper(remote_refs):
+            if self._refs is not None:
+                update_refs_container(self._refs, remote_refs)
+            return get_changed_refs(remote_refs)
         try:
             return self._client.send_pack(
                 self._client_path, get_changed_refs_wrapper,
@@ -463,11 +464,10 @@
         refname = self._get_selected_ref(name, ref)
         if refname != b'HEAD' and refname in self.get_refs_container():
             raise AlreadyBranchError(self.user_url)
-        if refname in self.get_refs_container():
-            ref_chain, unused_sha = self.get_refs_container().follow(
-                self._get_selected_ref(None))
-            if ref_chain[0] == b'HEAD':
-                refname = ref_chain[1]
+        ref_chain, unused_sha = self.get_refs_container().follow(
+            self._get_selected_ref(name))
+        if ref_chain and ref_chain[0] == b'HEAD':
+            refname = ref_chain[1]
         repo = self.open_repository()
         return RemoteGitBranch(self, repo, refname)
 
@@ -562,12 +562,18 @@
         push_result.branch_push_result = None
         repo = self.find_repository()
         refname = self._get_selected_ref(name)
+        ref_chain, old_sha = self.get_refs_container().follow(refname)
+        if ref_chain:
+            actual_refname = ref_chain[-1]
+        else:
+            actual_refname = refname
         if isinstance(source, GitBranch) and lossy:
             raise errors.LossyPushToSameVCS(source.controldir, self)
         source_store = get_object_store(source.repository)
         fetch_tags = source.get_config_stack().get('branch.fetch_tags')
-        def get_changed_refs(refs):
-            self._refs = remote_refs_dict_to_container(refs)
+        def get_changed_refs(remote_refs):
+            if self._refs is not None:
+                update_refs_container(self._refs, remote_refs)
             ret = {}
             # TODO(jelmer): Unpeel if necessary
             push_result.new_original_revid = revision_id
@@ -580,15 +586,18 @@
                     raise errors.NoRoundtrippingSupport(
                         source, self.open_branch(name=name, nascent_ok=True))
             if not overwrite:
-                if remote_divergence(ret.get(refname), new_sha,
-                                     source_store):
+                if remote_divergence(old_sha, new_sha, source_store):
                     raise DivergedBranches(
                         source, self.open_branch(name, nascent_ok=True))
-            ret[refname] = new_sha
+            ret[actual_refname] = new_sha
             if fetch_tags:
                 for tagname, revid in source.tags.get_tag_dict().items():
                     if lossy:
-                        new_sha = source_store._lookup_revision_sha1(revid)
+                        try:
+                            new_sha = source_store._lookup_revision_sha1(revid)
+                        except KeyError:
+                            if source.repository.has_revision(revid):
+                                raise
                     else:
                         try:
                             new_sha = repo.lookup_bzr_revision_id(revid)[0]
@@ -603,15 +612,15 @@
                 generate_pack_data = source_store.generate_pack_data
             new_refs = self.send_pack(get_changed_refs, generate_pack_data)
         push_result.new_revid = repo.lookup_foreign_revision_id(
-            new_refs[refname])
-        try:
-            old_remote = self._refs[refname]
-        except KeyError:
-            old_remote = ZERO_SHA
-        push_result.old_revid = repo.lookup_foreign_revision_id(old_remote)
-        self._refs = remote_refs_dict_to_container(new_refs)
+            new_refs[actual_refname])
+        if old_sha is not None:
+            push_result.old_revid = repo.lookup_foreign_revision_id(old_sha)
+        else:
+            push_result.old_revid = NULL_REVISION
+        if self._refs is not None:
+            update_refs_container(self._refs, new_refs)
         push_result.target_branch = self.open_branch(name)
-        if old_remote != ZERO_SHA:
+        if old_sha is not None:
             push_result.branch_push_result = GitBranchPushResult()
             push_result.branch_push_result.source_branch = source
             push_result.branch_push_result.target_branch = (
@@ -1021,3 +1030,15 @@
     ret = DictRefsContainer(base)
     ret._peeled = peeled
     return ret
+
+
+def update_refs_container(container, refs_dict):
+    peeled = {}
+    base = {}
+    for k, v in refs_dict.items():
+        if is_peeled(k):
+            peeled[k[:-3]] = v
+        else:
+            base[k] = v
+    container._peeled = peeled
+    container._refs.update(base)

=== modified file 'breezy/git/tests/test_remote.py'
--- breezy/git/tests/test_remote.py	2019-11-19 18:10:28 +0000
+++ breezy/git/tests/test_remote.py	2020-02-08 18:51:59 +0000
@@ -357,6 +357,42 @@
              },
             self.remote_real.get_refs())
 
+    def test_push_branch_symref(self):
+        cfg = self.remote_real.get_config()
+        cfg.set((b'core', ), b'bare', True)
+        cfg.write_to_path()
+        self.remote_real.refs.set_symbolic_ref(b'HEAD', b'refs/heads/master')
+        c1 = self.remote_real.do_commit(
+            message=b'message',
+            committer=b'committer <committer@example.com>',
+            author=b'author <author@example.com>',
+            ref=b'refs/heads/master')
+        remote = ControlDir.open(self.remote_url)
+        wt = self.make_branch_and_tree('local', format=self._from_format)
+        self.build_tree(['local/blah'])
+        wt.add(['blah'])
+        revid = wt.commit('blah')
+
+        if self._from_format == 'git':
+            result = remote.push_branch(wt.branch, overwrite=True)
+        else:
+            result = remote.push_branch(wt.branch, lossy=True, overwrite=True)
+
+        self.assertEqual(None, result.old_revno)
+        if self._from_format == 'git':
+            self.assertEqual(1, result.new_revno)
+        else:
+            self.assertIs(None, result.new_revno)
+
+        result.report(BytesIO())
+
+        self.assertEqual(
+            {
+                b'HEAD': self.remote_real.refs[b'refs/heads/master'],
+                b'refs/heads/master': self.remote_real.refs[b'refs/heads/master'],
+            },
+            self.remote_real.get_refs())
+
     def test_push_branch_new_with_tags(self):
         remote = ControlDir.open(self.remote_url)
         builder = self.make_branch_builder('local', format=self._from_format)

=== modified file 'breezy/git/tree.py'
--- breezy/git/tree.py	2020-02-07 02:14:30 +0000
+++ breezy/git/tree.py	2020-02-13 23:57:28 +0000
@@ -981,6 +981,50 @@
                           want_unversioned=False):
         raise NotImplementedError(self._iter_git_changes)
 
+    def find_target_path(self, path, recurse='none'):
+        ret = self.find_target_paths([path], recurse=recurse)
+        return ret[path]
+
+    def find_source_path(self, path, recurse='none'):
+        ret = self.find_source_paths([path], recurse=recurse)
+        return ret[path]
+
+    def find_target_paths(self, paths, recurse='none'):
+        paths = set(paths)
+        ret = {}
+        changes = self._iter_git_changes(specific_files=paths)[0]
+        for (oldpath, newpath), (oldmode, newmode), (oldsha, newsha) in changes:
+            if oldpath in paths:
+                ret[oldpath] = newpath
+        for path in paths:
+            if path not in ret:
+                if self.source.has_filename(path):
+                    if self.target.has_filename(path):
+                        ret[path] = path
+                    else:
+                        ret[path] = None
+                else:
+                    raise errors.NoSuchFile(path)
+        return ret
+
+    def find_source_paths(self, paths, recurse='none'):
+        paths = set(paths)
+        ret = {}
+        changes = self._iter_git_changes(specific_files=paths)[0]
+        for (oldpath, newpath), (oldmode, newmode), (oldsha, newsha) in changes:
+            if newpath in paths:
+                ret[newpath] = oldpath
+        for path in paths:
+            if path not in ret:
+                if self.target.has_filename(path):
+                    if self.source.has_filename(path):
+                        ret[path] = path
+                    else:
+                        ret[path] = None
+                else:
+                    raise errors.NoSuchFile(path)
+        return ret
+
 
 class InterGitRevisionTrees(InterGitTrees):
     """InterTree that works between two git revision trees."""

=== modified file 'breezy/log.py'
--- breezy/log.py	2020-02-07 02:14:30 +0000
+++ breezy/log.py	2020-02-13 23:57:28 +0000
@@ -85,7 +85,10 @@
     get_terminal_encoding,
     terminal_width,
     )
-from .tree import find_previous_path
+from .tree import (
+    find_previous_path,
+    InterTree,
+    )
 
 
 def find_touching_revisions(repository, last_revision, last_tree, last_path):
@@ -105,7 +108,8 @@
     revno = len(history)
     for revision_id in history:
         this_tree = repository.revision_tree(revision_id)
-        this_path = find_previous_path(last_tree, this_tree, last_path)
+        this_intertree = InterTree.get(this_tree, last_tree)
+        this_path = this_intertree.find_source_path(last_path)
 
         # now we know how it was last time, and how it is in this revision.
         # are those two states effectively the same or not?

=== modified file 'breezy/tests/__init__.py'
--- breezy/tests/__init__.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/__init__.py	2020-02-13 23:57:28 +0000
@@ -3578,7 +3578,7 @@
         pid = os.fork()
         if pid == 0:
             try:
-                stream = os.fdopen(c2pwrite, 'wb', 1)
+                stream = os.fdopen(c2pwrite, 'wb', 0)
                 workaround_zealous_crypto_random()
                 try:
                     import coverage
@@ -3611,7 +3611,7 @@
             os._exit(0)
         else:
             os.close(c2pwrite)
-            stream = os.fdopen(c2pread, 'rb', 1)
+            stream = os.fdopen(c2pread, 'rb', 0)
             test = TestInOtherProcess(stream, pid)
             result.append(test)
     return result

=== modified file 'breezy/tests/blackbox/test_locale.py'
--- breezy/tests/blackbox/test_locale.py	2018-11-22 02:01:59 +0000
+++ breezy/tests/blackbox/test_locale.py	2020-02-07 00:50:28 +0000
@@ -69,6 +69,9 @@
   Unicode \xc2\xb5 commit
 """, out)
 
+    @unittest.skipIf(
+        sys.version_info[:2] >= (3, 8),
+        "python > 3.8 doesn't allow changing filesystem default encoding")
     def test_log_C(self):
         self.disable_missing_extensions_warning()
         out, err = self.run_log_quiet_long(
@@ -90,6 +93,9 @@
   Unicode ? commit
 """, out)
 
+    @unittest.skipIf(
+        sys.version_info[:2] >= (3, 8),
+        "python > 3.8 doesn't allow changing filesystem default encoding")
     def test_log_BOGUS(self):
         out, err = self.run_log_quiet_long(
             ['tree'],

=== modified file 'breezy/tests/matchers.py'
--- breezy/tests/matchers.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/matchers.py	2020-02-13 23:57:28 +0000
@@ -45,7 +45,10 @@
 from breezy.bzr.smart.request import request_handlers as smart_request_handlers
 from breezy.bzr.smart import vfs
 """)
-from ..tree import find_previous_path
+from ..tree import (
+    find_previous_path,
+    InterTree,
+    )
 
 from testtools.matchers import Equals, Mismatch, Matcher
 
@@ -198,11 +201,11 @@
 
     def get_path_map(self, tree):
         """Get the (path, previous_path) pairs for the current tree."""
+        previous_intertree = InterTree.get(self.previous_tree, tree)
         with tree.lock_read(), self.previous_tree.lock_read():
             for path, ie in tree.iter_entries_by_dir():
                 if tree.supports_rename_tracking():
-                    previous_path = find_previous_path(
-                        tree, self.previous_tree, path)
+                    previous_path = previous_intertree.find_source_path(path)
                 else:
                     if self.previous_tree.is_versioned(path):
                         previous_path = path

=== modified file 'breezy/tests/per_intertree/__init__.py'
--- breezy/tests/per_intertree/__init__.py	2018-11-11 04:08:32 +0000
+++ breezy/tests/per_intertree/__init__.py	2020-02-08 22:40:33 +0000
@@ -127,6 +127,7 @@
     submod_tests = loader.loadTestsFromModuleNames([
         'breezy.tests.per_intertree.test_compare',
         'breezy.tests.per_intertree.test_file_content_matches',
+        'breezy.tests.per_intertree.test_find_path',
         ])
     test_intertree_permutations = [
         # test InterTree with two default-format working trees.

=== added file 'breezy/tests/per_intertree/test_find_path.py'
--- breezy/tests/per_intertree/test_find_path.py	1970-01-01 00:00:00 +0000
+++ breezy/tests/per_intertree/test_find_path.py	2020-02-08 22:40:33 +0000
@@ -0,0 +1,126 @@
+# Copyright (C) 2020 Jelmer Vernooij
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for breezy.branch.InterBranch.copy_content_into."""
+
+from breezy import branch
+from breezy.errors import NoSuchFile
+from breezy.tests import TestNotApplicable
+from breezy.tests.per_intertree import (
+    TestCaseWithTwoTrees,
+    )
+
+
+class TestFindPaths(TestCaseWithTwoTrees):
+
+    def test_path_missing(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        self.build_tree_contents([
+            ('1/file', b'apples'),
+            ('2/file', b'apples'),
+            ])
+        tree1.add('file')
+        tree2.add('file')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertRaises(NoSuchFile, inter.find_source_path, 'missing')
+        self.assertRaises(NoSuchFile, inter.find_target_path, 'missing')
+        self.assertRaises(NoSuchFile, inter.find_source_paths, ['missing'])
+        self.assertRaises(NoSuchFile, inter.find_target_paths, ['missing'])
+
+    def test_old_path(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        self.build_tree_contents([
+            ('2/file', b'apples'),
+            ])
+        tree2.add('file')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertIs(None, inter.find_source_path('file'))
+        self.assertEqual({'file': None}, inter.find_source_paths(['file']))
+
+    def test_new_path(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        self.build_tree_contents([
+            ('1/file', b'apples'),
+            ])
+        tree1.add('file')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertIs(None, inter.find_target_path('file'))
+        self.assertEqual({'file': None}, inter.find_target_paths(['file']))
+
+    def test_unchanged(self):
+        tree1 = self.make_branch_and_tree('1')
+        self.build_tree_contents([
+            ('1/file', b'apples'),
+            ])
+        tree1.add('file')
+        tree1.commit('foo')
+
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.pull(tree1.branch)
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertEqual('file', inter.find_target_path('file'))
+        self.assertEqual({'file': 'file'}, inter.find_target_paths(['file']))
+        self.assertEqual('file', inter.find_source_path('file'))
+        self.assertEqual({'file': 'file'}, inter.find_source_paths(['file']))
+
+    def test_rename(self):
+        tree1 = self.make_branch_and_tree('1')
+        self.build_tree_contents([
+            ('1/file', b'apples'),
+            ])
+        tree1.add('file')
+        tree1.commit('foo')
+
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.pull(tree1.branch)
+        tree2.rename_one('file', 'newfile')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertEqual('newfile', inter.find_target_path('file'))
+        self.assertEqual({'file': 'newfile'}, inter.find_target_paths(['file']))
+        self.assertEqual('file', inter.find_source_path('newfile'))
+        self.assertEqual({'newfile': 'file'}, inter.find_source_paths(['newfile']))
+
+    def test_copy(self):
+        tree1 = self.make_branch_and_tree('1')
+        self.build_tree_contents([
+            ('1/file', b'apples'),
+            ])
+        tree1.add('file')
+        tree1.commit('foo')
+
+        tree2 = self.make_to_branch_and_tree('2')
+        tree2.pull(tree1.branch)
+        tree2.copy_one('file', 'newfile')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertIn(inter.find_target_path('file'), ['file', 'newfile'])
+        self.assertIn(inter.find_source_path('newfile'), ['file', None])
+        self.assertEqual('file', inter.find_source_path('file'))
+        self.assertIn(inter.find_source_paths(['newfile']), [{'newfile': 'file'}, {'newfile': None}])
+
+        self.assertIn(
+            inter.find_source_paths(['newfile', 'file']), [
+                {'newfile': 'file', 'file': 'file'},
+                {'newfile': None, 'file': 'file'},
+                ])

=== modified file 'breezy/tests/test_bundle.py'
--- breezy/tests/test_bundle.py	2020-01-30 16:13:31 +0000
+++ breezy/tests/test_bundle.py	2020-02-09 17:46:22 +0000
@@ -43,7 +43,7 @@
     features,
     test_commit,
     )
-from ..tree import find_previous_path
+from ..tree import InterTree
 
 
 def get_text(vf, key):
@@ -562,7 +562,7 @@
 
         for path, status, kind, entry in base_files:
             # Check that the meta information is the same
-            to_path = find_previous_path(base_tree, to_tree, path)
+            to_path = InterTree.get(base_tree, to_tree).find_target_path(path)
             self.assertEqual(
                 base_tree.get_file_size(path),
                 to_tree.get_file_size(to_path))
@@ -1796,7 +1796,7 @@
         writer = v4.BundleWriter(fileobj)
         writer.begin()
         writer.add_info_record({b'foo': b'bar'})
-        writer._container.add_bytes_record(b'blah', [(b'two', ), (b'names', )])
+        writer._container.add_bytes_record([b'blah'], len(b'blah'), [(b'two', ), (b'names', )])
         writer.end()
         fileobj.seek(0)
         record_iter = v4.BundleReader(fileobj).iter_records()

=== modified file 'breezy/tests/test_export_pot.py'
--- breezy/tests/test_export_pot.py	2018-11-11 04:08:32 +0000
+++ breezy/tests/test_export_pot.py	2020-02-07 00:50:28 +0000
@@ -15,6 +15,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import re
+import sys
 import textwrap
 
 from io import StringIO
@@ -149,11 +150,13 @@
     )
 '''
         _, str_lines = export_pot._parse_source(src)
-        self.expectFailure("Escaped newlines confuses the multiline handling",
-                           self.assertNotEqual, str_lines,
-                           {"Escaped\n": 0, "Raw\\n": 2, "A\n\nB\n\nC\n\n": -2})
-        self.assertEqual(str_lines,
-                         {"Escaped\n": 1, "Raw\\n": 2, "A\n\nB\n\nC\n\n": 4})
+        if sys.version_info < (3, 8):
+            self.expectFailure("Escaped newlines confuses the multiline handling",
+                               self.assertNotEqual, str_lines,
+                               {"Escaped\n": 0, "Raw\\n": 2, "A\n\nB\n\nC\n\n": -2})
+        else:
+            self.assertEqual(
+                str_lines, {"Escaped\n": 1, "Raw\\n": 2, "A\n\nB\n\nC\n\n": 4})
 
 
 class TestModuleContext(tests.TestCase):

=== modified file 'breezy/tests/test_groupcompress.py'
--- breezy/tests/test_groupcompress.py	2020-01-26 04:30:31 +0000
+++ breezy/tests/test_groupcompress.py	2020-02-09 02:20:11 +0000
@@ -124,12 +124,12 @@
         sha1_2, _, end_point, _ = compressor.compress(
             ('newlabel',), [text], len(text), None)
         # get the first out
-        self.assertEqual((b'strange\ncommon long line\n'
-                          b'that needs a 16 byte match\n', sha1_1),
+        self.assertEqual(([b'strange\ncommon long line\n'
+                           b'that needs a 16 byte match\n'], sha1_1),
                          compressor.extract(('label',)))
         # and the second
-        self.assertEqual((b'common long line\nthat needs a 16 byte match\n'
-                          b'different\n', sha1_2),
+        self.assertEqual(([b'common long line\nthat needs a 16 byte match\n'
+                           b'different\n'], sha1_2),
                          compressor.extract(('newlabel',)))
 
     def test_pop_last(self):

=== modified file 'breezy/tests/test_inventory_delta.py'
--- breezy/tests/test_inventory_delta.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/test_inventory_delta.py	2020-02-13 23:57:28 +0000
@@ -22,6 +22,7 @@
 from io import (
     BytesIO,
     )
+from .. import osutils
 from ..bzr import (
     inventory,
     inventory_delta,
@@ -96,28 +97,28 @@
     def test_parse_no_bytes(self):
         deserializer = inventory_delta.InventoryDeltaDeserializer()
         err = self.assertRaises(
-            InventoryDeltaError, deserializer.parse_text_bytes, b'')
-        self.assertContainsRe(str(err), 'last line not empty')
+            InventoryDeltaError, deserializer.parse_text_bytes, [])
+        self.assertContainsRe(str(err), 'inventory delta is empty')
 
     def test_parse_bad_format(self):
         deserializer = inventory_delta.InventoryDeltaDeserializer()
         err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, b'format: foo\n')
+                                deserializer.parse_text_bytes, [b'format: foo\n'])
         self.assertContainsRe(str(err), 'unknown format')
 
     def test_parse_no_parent(self):
         deserializer = inventory_delta.InventoryDeltaDeserializer()
         err = self.assertRaises(InventoryDeltaError,
                                 deserializer.parse_text_bytes,
-                                b'format: bzr inventory delta v1 (bzr 1.14)\n')
+                                [b'format: bzr inventory delta v1 (bzr 1.14)\n'])
         self.assertContainsRe(str(err), 'missing parent: marker')
 
     def test_parse_no_version(self):
         deserializer = inventory_delta.InventoryDeltaDeserializer()
         err = self.assertRaises(InventoryDeltaError,
                                 deserializer.parse_text_bytes,
-                                b'format: bzr inventory delta v1 (bzr 1.14)\n'
-                                b'parent: null:\n')
+                                [b'format: bzr inventory delta v1 (bzr 1.14)\n',
+                                 b'parent: null:\n'])
         self.assertContainsRe(str(err), 'missing version: marker')
 
     def test_parse_duplicate_key_errors(self):
@@ -132,12 +133,12 @@
 None\x00/\x00an-id\x00\x00a@e\xc3\xa5ample.com--2004\x00dir\x00\x00
 """
         err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, double_root_lines)
+                                deserializer.parse_text_bytes, osutils.split_lines(double_root_lines))
         self.assertContainsRe(str(err), 'duplicate file id')
 
     def test_parse_versioned_root_only(self):
         deserializer = inventory_delta.InventoryDeltaDeserializer()
-        parse_result = deserializer.parse_text_bytes(root_only_lines)
+        parse_result = deserializer.parse_text_bytes(osutils.split_lines(root_only_lines))
         expected_entry = inventory.make_entry(
             'directory', u'', None, b'an-id')
         expected_entry.revision = b'a@e\xc3\xa5ample.com--2004'
@@ -155,8 +156,9 @@
 tree_references: true
 None\x00/\x00TREE_ROOT\x00\x00null:\x00dir\x00\x00
 """
-        err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, root_only_lines)
+        err = self.assertRaises(
+            InventoryDeltaError, deserializer.parse_text_bytes,
+            osutils.split_lines(root_only_lines))
         self.assertContainsRe(str(err), 'special revisionid found')
 
     def test_parse_versioned_root_versioned_disabled(self):
@@ -168,8 +170,9 @@
 tree_references: true
 None\x00/\x00TREE_ROOT\x00\x00a@e\xc3\xa5ample.com--2004\x00dir\x00\x00
 """
-        err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, root_only_lines)
+        err = self.assertRaises(
+            InventoryDeltaError, deserializer.parse_text_bytes,
+            osutils.split_lines(root_only_lines))
         self.assertContainsRe(str(err), 'Versioned root found')
 
     def test_parse_unique_root_id_root_versioned_disabled(self):
@@ -182,12 +185,13 @@
 None\x00/\x00an-id\x00\x00parent-id\x00dir\x00\x00
 """
         err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, root_only_lines)
+                                deserializer.parse_text_bytes, osutils.split_lines(root_only_lines))
         self.assertContainsRe(str(err), 'Versioned root found')
 
     def test_parse_unversioned_root_versioning_enabled(self):
         deserializer = inventory_delta.InventoryDeltaDeserializer()
-        parse_result = deserializer.parse_text_bytes(root_only_unversioned)
+        parse_result = deserializer.parse_text_bytes(
+            osutils.split_lines(root_only_unversioned))
         expected_entry = inventory.make_entry(
             'directory', u'', None, b'TREE_ROOT')
         expected_entry.revision = b'entry-version'
@@ -200,14 +204,14 @@
         deserializer = inventory_delta.InventoryDeltaDeserializer(
             allow_versioned_root=False)
         err = self.assertRaises(inventory_delta.IncompatibleInventoryDelta,
-                                deserializer.parse_text_bytes, root_only_lines)
+                                deserializer.parse_text_bytes, osutils.split_lines(root_only_lines))
         self.assertEqual("versioned_root not allowed", str(err))
 
     def test_parse_tree_when_disabled(self):
         deserializer = inventory_delta.InventoryDeltaDeserializer(
             allow_tree_references=False)
         err = self.assertRaises(inventory_delta.IncompatibleInventoryDelta,
-                                deserializer.parse_text_bytes, reference_lines)
+                                deserializer.parse_text_bytes, osutils.split_lines(reference_lines))
         self.assertEqual("Tree reference not allowed", str(err))
 
     def test_parse_tree_when_header_disallows(self):
@@ -223,7 +227,7 @@
 None\x00/foo\x00id\x00TREE_ROOT\x00changed\x00tree\x00subtree-version
 """
         err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, lines)
+                                deserializer.parse_text_bytes, osutils.split_lines(lines))
         self.assertContainsRe(str(err), 'Tree reference found')
 
     def test_parse_versioned_root_when_header_disallows(self):
@@ -239,7 +243,7 @@
 None\x00/\x00TREE_ROOT\x00\x00a@e\xc3\xa5ample.com--2004\x00dir
 """
         err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, lines)
+                                deserializer.parse_text_bytes, osutils.split_lines(lines))
         self.assertContainsRe(str(err), 'Versioned root found')
 
     def test_parse_last_line_not_empty(self):
@@ -248,7 +252,7 @@
         lines = root_only_lines[:-1]
         deserializer = inventory_delta.InventoryDeltaDeserializer()
         err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, lines)
+                                deserializer.parse_text_bytes, osutils.split_lines(lines))
         self.assertContainsRe(str(err), 'last line not empty')
 
     def test_parse_invalid_newpath(self):
@@ -257,7 +261,7 @@
         lines += b"None\x00bad\x00TREE_ROOT\x00\x00version\x00dir\n"
         deserializer = inventory_delta.InventoryDeltaDeserializer()
         err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, lines)
+                                deserializer.parse_text_bytes, osutils.split_lines(lines))
         self.assertContainsRe(str(err), 'newpath invalid')
 
     def test_parse_invalid_oldpath(self):
@@ -266,7 +270,7 @@
         lines += b"bad\x00/new\x00file-id\x00\x00version\x00dir\n"
         deserializer = inventory_delta.InventoryDeltaDeserializer()
         err = self.assertRaises(InventoryDeltaError,
-                                deserializer.parse_text_bytes, lines)
+                                deserializer.parse_text_bytes, osutils.split_lines(lines))
         self.assertContainsRe(str(err), 'oldpath invalid')
 
     def test_parse_new_file(self):
@@ -277,7 +281,7 @@
             b"None\x00/new\x00file-id\x00an-id\x00version\x00file\x00123\x00" +
             b"\x00" + fake_sha + b"\n")
         deserializer = inventory_delta.InventoryDeltaDeserializer()
-        parse_result = deserializer.parse_text_bytes(lines)
+        parse_result = deserializer.parse_text_bytes(osutils.split_lines(lines))
         expected_entry = inventory.make_entry(
             'file', u'new', b'an-id', b'file-id')
         expected_entry.revision = b'version'
@@ -292,7 +296,7 @@
         lines += (
             b"/old-file\x00None\x00deleted-id\x00\x00null:\x00deleted\x00\x00\n")
         deserializer = inventory_delta.InventoryDeltaDeserializer()
-        parse_result = deserializer.parse_text_bytes(lines)
+        parse_result = deserializer.parse_text_bytes(osutils.split_lines(lines))
         delta = parse_result[4]
         self.assertEqual(
             (u'old-file', None, b'deleted-id', None), delta[-1])
@@ -339,7 +343,7 @@
         deserializer = inventory_delta.InventoryDeltaDeserializer()
         self.assertEqual(
             (NULL_REVISION, b'entry-version', False, False, delta),
-            deserializer.parse_text_bytes(b''.join(serialized_lines)))
+            deserializer.parse_text_bytes(serialized_lines))
 
     def test_unversioned_non_root_errors(self):
         old_inv = Inventory(None)

=== modified file 'breezy/tests/test_pack.py'
--- breezy/tests/test_pack.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/test_pack.py	2020-02-13 23:57:28 +0000
@@ -118,14 +118,14 @@
     def test_non_empty_end_does_not_add_a_record_to_records_written(self):
         """The end() method does not count towards the records written."""
         self.writer.begin()
-        self.writer.add_bytes_record(b'foo', names=[])
+        self.writer.add_bytes_record([b'foo'], len(b'foo'), names=[])
         self.writer.end()
         self.assertEqual(1, self.writer.records_written)
 
     def test_add_bytes_record_no_name(self):
         """Add a bytes record with no name."""
         self.writer.begin()
-        offset, length = self.writer.add_bytes_record(b'abc', names=[])
+        offset, length = self.writer.add_bytes_record([b'abc'], len(b'abc'), names=[])
         self.assertEqual((42, 7), (offset, length))
         self.assertOutput(
             b'Bazaar pack format 1 (introduced in 0.18)\nB3\n\nabc')
@@ -135,7 +135,7 @@
         self.writer.begin()
 
         offset, length = self.writer.add_bytes_record(
-            b'abc', names=[(b'name1', )])
+            [b'abc'], len(b'abc'), names=[(b'name1', )])
         self.assertEqual((42, 13), (offset, length))
         self.assertOutput(
             b'Bazaar pack format 1 (introduced in 0.18)\n'
@@ -156,7 +156,7 @@
 
         self.writer.begin()
         offset, length = self.writer.add_bytes_record(
-            b'abcabc', names=[(b'name1', )])
+            [b'abcabc'], len(b'abcabc'), names=[(b'name1', )])
         self.assertEqual((42, 16), (offset, length))
         self.assertOutput(
             b'Bazaar pack format 1 (introduced in 0.18)\n'
@@ -172,7 +172,7 @@
         """Add a bytes record with two names."""
         self.writer.begin()
         offset, length = self.writer.add_bytes_record(
-            b'abc', names=[(b'name1', ), (b'name2', )])
+            [b'abc'], len(b'abc'), names=[(b'name1', ), (b'name2', )])
         self.assertEqual((42, 19), (offset, length))
         self.assertOutput(
             b'Bazaar pack format 1 (introduced in 0.18)\n'
@@ -182,7 +182,7 @@
         """Add a bytes record with two names."""
         self.writer.begin()
         offset, length = self.writer.add_bytes_record(
-            b'abc', names=[(b'name1', ), (b'name2', )])
+            [b'abc'], len(b'abc'), names=[(b'name1', ), (b'name2', )])
         self.assertEqual((42, 19), (offset, length))
         self.assertOutput(
             b'Bazaar pack format 1 (introduced in 0.18)\n'
@@ -192,7 +192,7 @@
         """Add a bytes record with a two-element name."""
         self.writer.begin()
         offset, length = self.writer.add_bytes_record(
-            b'abc', names=[(b'name1', b'name2')])
+            [b'abc'], len(b'abc'), names=[(b'name1', b'name2')])
         self.assertEqual((42, 19), (offset, length))
         self.assertOutput(
             b'Bazaar pack format 1 (introduced in 0.18)\n'
@@ -200,8 +200,8 @@
 
     def test_add_second_bytes_record_gets_higher_offset(self):
         self.writer.begin()
-        self.writer.add_bytes_record(b'abc', names=[])
-        offset, length = self.writer.add_bytes_record(b'abc', names=[])
+        self.writer.add_bytes_record([b'a', b'bc'], len(b'abc'), names=[])
+        offset, length = self.writer.add_bytes_record([b'abc'], len(b'abc'), names=[])
         self.assertEqual((49, 7), (offset, length))
         self.assertOutput(
             b'Bazaar pack format 1 (introduced in 0.18)\n'
@@ -215,14 +215,14 @@
         self.writer.begin()
         self.assertRaises(
             errors.InvalidRecordError,
-            self.writer.add_bytes_record, b'abc', names=[(b'bad name', )])
+            self.writer.add_bytes_record, [b'abc'], len(b'abc'), names=[(b'bad name', )])
 
     def test_add_bytes_records_add_to_records_written(self):
         """Adding a Bytes record increments the records_written counter."""
         self.writer.begin()
-        self.writer.add_bytes_record(b'foo', names=[])
+        self.writer.add_bytes_record([b'foo'], len(b'foo'), names=[])
         self.assertEqual(1, self.writer.records_written)
-        self.writer.add_bytes_record(b'foo', names=[])
+        self.writer.add_bytes_record([b'foo'], len(b'foo'), names=[])
         self.assertEqual(2, self.writer.records_written)
 
 
@@ -538,10 +538,10 @@
         writer = pack.ContainerWriter(pack_data.write)
         writer.begin()
         memos = []
-        memos.append(writer.add_bytes_record(b'abc', names=[]))
-        memos.append(writer.add_bytes_record(b'def', names=[(b'name1', )]))
-        memos.append(writer.add_bytes_record(b'ghi', names=[(b'name2', )]))
-        memos.append(writer.add_bytes_record(b'jkl', names=[]))
+        memos.append(writer.add_bytes_record([b'abc'], 3, names=[]))
+        memos.append(writer.add_bytes_record([b'def'], 3, names=[(b'name1', )]))
+        memos.append(writer.add_bytes_record([b'ghi'], 3, names=[(b'name2', )]))
+        memos.append(writer.add_bytes_record([b'jkl'], 3, names=[]))
         writer.end()
         transport = self.get_transport()
         transport.put_bytes('mypack', pack_data.getvalue())

=== modified file 'breezy/transform.py'
--- breezy/transform.py	2020-02-07 02:14:30 +0000
+++ breezy/transform.py	2020-02-13 23:57:28 +0000
@@ -66,7 +66,7 @@
     )
 from .progress import ProgressPhase
 from .tree import (
-    find_previous_path,
+    InterTree,
     TreeChange,
     )
 
@@ -2956,8 +2956,8 @@
                         if basis_tree is None:
                             basis_tree = working_tree.basis_tree()
                             basis_tree.lock_read()
-                        basis_path = find_previous_path(
-                            working_tree, basis_tree, wt_path)
+                        basis_inter = InterTree.get(basis_tree, working_tree)
+                        basis_path = basis_inter.find_source_path(wt_path)
                         if basis_path is None:
                             if target_kind is None and not target_versioned:
                                 keep_content = True
@@ -2996,7 +2996,8 @@
                         basis_tree = working_tree.basis_tree()
                         basis_tree.lock_read()
                     new_sha1 = target_tree.get_file_sha1(target_path)
-                    basis_path = find_previous_path(target_tree, basis_tree, target_path)
+                    basis_inter = InterTree.get(basis_tree, target_tree)
+                    basis_path = basis_inter.find_source_path(target_path)
                     if (basis_path is not None and
                             new_sha1 == basis_tree.get_file_sha1(basis_path)):
                         # If the new contents of the file match what is in basis,

=== modified file 'breezy/tree.py'
--- breezy/tree.py	2020-02-07 02:14:30 +0000
+++ breezy/tree.py	2020-02-13 23:57:28 +0000
@@ -709,8 +709,7 @@
         :return: None if content filtering is not supported by this tree.
         """
         if self.supports_content_filtering():
-            return lambda path, file_id: \
-                self._content_filter_stack(path)
+            return self._content_filter_stack
         else:
             return None
 
@@ -963,8 +962,7 @@
         from_data = dict(from_entries_by_dir)
         to_entries_by_dir = list(self.target.iter_entries_by_dir(
             specific_files=target_specific_files))
-        path_equivs = find_previous_paths(
-            self.target, self.source, [p for p, e in to_entries_by_dir])
+        path_equivs = self.find_source_paths([p for p, e in to_entries_by_dir])
         num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
         entry_count = 0
         # the unversioned path lookup only occurs on real trees - where there
@@ -1027,7 +1025,7 @@
             if file_id in to_paths:
                 # already returned
                 continue
-            to_path = find_previous_path(self.source, self.target, path)
+            to_path = self.find_target_path(path)
             entry_count += 1
             if pb is not None:
                 pb.update('comparing files', entry_count, num_entries)
@@ -1191,6 +1189,60 @@
                 target_sha1 = target_verifier_data
             return (source_sha1 == target_sha1)
 
+    def find_target_path(self, path, recurse='none'):
+        """Find target tree path.
+
+        :param path: Path to search for (exists in source)
+        :return: path in target, or None if there is no equivalent path.
+        :raise NoSuchFile: If the path doesn't exist in source
+        """
+        file_id = self.source.path2id(path)
+        if file_id is None:
+            raise errors.NoSuchFile(path)
+        try:
+            return self.target.id2path(file_id, recurse=recurse)
+        except errors.NoSuchId:
+            return None
+
+    def find_source_path(self, path, recurse='none'):
+        """Find the source tree path.
+
+        :param path: Path to search for (exists in target)
+        :return: path in source, or None if there is no equivalent path.
+        :raise NoSuchFile: if the path doesn't exist in target
+        """
+        file_id = self.target.path2id(path)
+        if file_id is None:
+            raise errors.NoSuchFile(path)
+        try:
+            return self.source.id2path(file_id, recurse=recurse)
+        except errors.NoSuchId:
+            return None
+
+    def find_target_paths(self, paths, recurse='none'):
+        """Find target tree paths.
+
+        :param paths: Iterable over paths in target to search for
+        :return: Dictionary mapping from source paths to paths in target , or
+            None if there is no equivalent path.
+        """
+        ret = {}
+        for path in paths:
+            ret[path] = self.find_target_path(path, recurse=recurse)
+        return ret
+
+    def find_source_paths(self, paths, recurse='none'):
+        """Find source tree paths.
+
+        :param paths: Iterable over paths in target to search for
+        :return: Dictionary mapping from target paths to paths in source, or
+            None if there is no equivalent path.
+        """
+        ret = {}
+        for path in paths:
+            ret[path] = self.find_source_path(path, recurse=recurse)
+        return ret
+
 
 InterTree.register_optimiser(InterTree)
 
@@ -1204,13 +1256,10 @@
     :return: Dictionary mapping from from_tree paths to paths in to_tree, or
         None if there is no equivalent path.
     """
-    ret = {}
-    for path in paths:
-        ret[path] = find_previous_path(from_tree, to_tree, path, recurse=recurse)
-    return ret
-
-
-def find_previous_path(from_tree, to_tree, path, file_id=None, recurse='none'):
+    return InterTree.get(to_tree, from_tree).find_source_paths(paths, recurse=recurse)
+
+
+def find_previous_path(from_tree, to_tree, path, recurse='none'):
     """Find previous tree path.
 
     :param from_tree: From tree
@@ -1219,14 +1268,8 @@
     :return: path in to_tree, or None if there is no equivalent path.
     :raise NoSuchFile: If the path doesn't exist in from_tree
     """
-    if file_id is None:
-        file_id = from_tree.path2id(path)
-    if file_id is None:
-        raise errors.NoSuchFile(path)
-    try:
-        return to_tree.id2path(file_id, recurse=recurse)
-    except errors.NoSuchId:
-        return None
+    return InterTree.get(to_tree, from_tree).find_source_path(
+        path, recurse=recurse)
 
 
 def get_canonical_path(tree, path, normalize):

=== modified file 'doc/en/release-notes/brz-3.1.txt'
--- doc/en/release-notes/brz-3.1.txt	2020-01-31 17:43:44 +0000
+++ doc/en/release-notes/brz-3.1.txt	2020-02-08 17:00:10 +0000
@@ -70,6 +70,9 @@
    attempt to access Fossil repositories.
    (Jelmer Vernooĳ, #1848821)
 
+ * When pushing to Git repositories, symrefs are now followed.
+   (Jelmer Vernooĳ, #1800393)
+
 Improvements
 ************
 

